////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Qualcomm Technologies, Inc. and/or its subsidiaries.
// All Rights Reserved.
// Confidential and Proprietary - Qualcomm Technologies, Inc.
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// THIS IS AUTO-GENERATED CODE.  PLEASE DO NOT EDIT (File bug reports against tools).
///
/// Auto-generated by: Parameter Parser V5.5.1 (2411131018)
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file  cvp_2_0_0.cpp
/// @brief Auto-generated Chromatix parameter file
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "cvp_2_0_0.h"
#include "parametersetmanager.h"

CVP_2_0_0_NAMESPACE_BEGIN

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// SetValue/GetValue Functions
/// Gets or sets the requested value
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_cvp20_reserveType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "image_based_alignment_enable") == 0)
    {
        ParameterUtils::DoubleToString((double)image_based_alignment_enable, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "descriptor_lpf") == 0)
    {
        ParameterUtils::DoubleToString((double)descriptor_lpf, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_cvp20_reserveType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "image_based_alignment_enable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            image_based_alignment_enable = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "descriptor_lpf") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            descriptor_lpf = (UINT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_cvp20_reserveType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // image_based_alignment_enable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("image_based_alignment_enable", ParameterParserDataType::UInt32, (double)image_based_alignment_enable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "image_based_alignment_enable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)image_based_alignment_enable, 0));
    }

    // descriptor_lpf
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("descriptor_lpf", ParameterParserDataType::UInt32, (double)descriptor_lpf, 0));
    }
    else if (PARAMETER_STRCMP(__name, "descriptor_lpf") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)descriptor_lpf, 0));
    }

    return __result;
}

bool cvp20_rgn_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "robustness_max_allowed_ncc") == 0)
    {
        ParameterUtils::DoubleToString((double)robustness_max_allowed_ncc, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "robustness_min_allowed_tar_var") == 0)
    {
        ParameterUtils::DoubleToString((double)robustness_min_allowed_tar_var, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "robustness_meaningful_ncc_diff") == 0)
    {
        ParameterUtils::DoubleToString((double)robustness_meaningful_ncc_diff, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "robustness_measure_dist_map_tab") == 0)
    {
        __result = robustness_measure_dist_map_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "fpx_kappa") == 0)
    {
        ParameterUtils::DoubleToString((double)fpx_kappa, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "fpx_threshold") == 0)
    {
        ParameterUtils::DoubleToString((double)fpx_threshold, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "desc_match_threshold") == 0)
    {
        ParameterUtils::DoubleToString((double)desc_match_threshold, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "robustness_threshold") == 0)
    {
        ParameterUtils::DoubleToString((double)robustness_threshold, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "inlier_track_enable") == 0)
    {
        ParameterUtils::DoubleToString((double)inlier_track_enable, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "enable_transform_confidence") == 0)
    {
        ParameterUtils::DoubleToString((double)enable_transform_confidence, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "transform_confidence_mapping_base") == 0)
    {
        ParameterUtils::DoubleToString((double)transform_confidence_mapping_base, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "transform_confidence_mapping_c1") == 0)
    {
        ParameterUtils::DoubleToString((double)transform_confidence_mapping_c1, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "transform_confidence_mapping_c2") == 0)
    {
        ParameterUtils::DoubleToString((double)transform_confidence_mapping_c2, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "transform_confidence_thr_to_force_identity_transform") == 0)
    {
        ParameterUtils::DoubleToString((double)transform_confidence_thr_to_force_identity_transform, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "transform_model") == 0)
    {
        ParameterUtils::DoubleToString((double)transform_model, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "model_match_thr") == 0)
    {
        ParameterUtils::DoubleToString((double)model_match_thr, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "multi_frame_input_resolution") == 0)
    {
        ParameterUtils::DoubleToString((double)multi_frame_input_resolution, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "video_registration_down_scale_ratio") == 0)
    {
        ParameterUtils::DoubleToString((double)video_registration_down_scale_ratio, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "auto_identity_detect_mode") == 0)
    {
        ParameterUtils::DoubleToString((double)auto_identity_detect_mode, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_enable") == 0)
    {
        ParameterUtils::DoubleToString((double)identity_detect_enable, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_consistency_mesure_th") == 0)
    {
        ParameterUtils::DoubleToString((double)identity_detect_consistency_mesure_th, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "identity_detection_mvs_avg_size_th") == 0)
    {
        ParameterUtils::DoubleToString((double)identity_detection_mvs_avg_size_th, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_big_mvs_percent_th") == 0)
    {
        ParameterUtils::DoubleToString((double)identity_detect_big_mvs_percent_th, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_temporal_consistency") == 0)
    {
        ParameterUtils::DoubleToString((double)identity_detect_temporal_consistency, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_complexity_th") == 0)
    {
        ParameterUtils::DoubleToString((double)identity_detect_complexity_th, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_max_num_mvs_th") == 0)
    {
        ParameterUtils::DoubleToString((double)identity_detect_max_num_mvs_th, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "confidence_auto_identity_detect") == 0)
    {
        ParameterUtils::DoubleToString((double)confidence_auto_identity_detect, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "motion_vector_screen_enable") == 0)
    {
        ParameterUtils::DoubleToString((double)motion_vector_screen_enable, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "motion_vector_screen_min_std_th") == 0)
    {
        ParameterUtils::DoubleToString((double)motion_vector_screen_min_std_th, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "motion_vector_screen_std_factor_th") == 0)
    {
        ParameterUtils::DoubleToString((double)motion_vector_screen_std_factor_th, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "conf1") == 0)
    {
        ParameterUtils::DoubleToString((double)conf1, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "conf2") == 0)
    {
        ParameterUtils::DoubleToString((double)conf2, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "conf3") == 0)
    {
        ParameterUtils::DoubleToString((double)conf3, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "conf4") == 0)
    {
        ParameterUtils::DoubleToString((double)conf4, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "conf5") == 0)
    {
        ParameterUtils::DoubleToString((double)conf5, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "contrast_gain_factor") == 0)
    {
        ParameterUtils::DoubleToString((double)contrast_gain_factor, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "reserved1") == 0)
    {
        ParameterUtils::DoubleToString((double)reserved1, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "reserved2") == 0)
    {
        ParameterUtils::DoubleToString((double)reserved2, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "reserved3") == 0)
    {
        ParameterUtils::DoubleToString((double)reserved3, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "reserved4") == 0)
    {
        ParameterUtils::DoubleToString((double)reserved4, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "reserved5") == 0)
    {
        ParameterUtils::DoubleToString((double)reserved5, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "ica_filtering") == 0)
    {
        ParameterUtils::DoubleToString((double)ica_filtering, __value, __size);
        __result = true;
    }

    return __result;
}

bool cvp20_rgn_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "robustness_max_allowed_ncc") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            robustness_max_allowed_ncc = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "robustness_min_allowed_tar_var") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            robustness_min_allowed_tar_var = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "robustness_meaningful_ncc_diff") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            robustness_meaningful_ncc_diff = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "robustness_measure_dist_map_tab") == 0)
    {
        __result = robustness_measure_dist_map_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "fpx_kappa") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            fpx_kappa = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "fpx_threshold") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            fpx_threshold = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "desc_match_threshold") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            desc_match_threshold = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "robustness_threshold") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            robustness_threshold = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "inlier_track_enable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            inlier_track_enable = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "enable_transform_confidence") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enable_transform_confidence = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "transform_confidence_mapping_base") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            transform_confidence_mapping_base = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "transform_confidence_mapping_c1") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            transform_confidence_mapping_c1 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "transform_confidence_mapping_c2") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            transform_confidence_mapping_c2 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "transform_confidence_thr_to_force_identity_transform") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            transform_confidence_thr_to_force_identity_transform = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "transform_model") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            transform_model = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "model_match_thr") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            model_match_thr = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "multi_frame_input_resolution") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            multi_frame_input_resolution = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "video_registration_down_scale_ratio") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            video_registration_down_scale_ratio = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "auto_identity_detect_mode") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            auto_identity_detect_mode = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_enable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            identity_detect_enable = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_consistency_mesure_th") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            identity_detect_consistency_mesure_th = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "identity_detection_mvs_avg_size_th") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            identity_detection_mvs_avg_size_th = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_big_mvs_percent_th") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            identity_detect_big_mvs_percent_th = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_temporal_consistency") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            identity_detect_temporal_consistency = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_complexity_th") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            identity_detect_complexity_th = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_max_num_mvs_th") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            identity_detect_max_num_mvs_th = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "confidence_auto_identity_detect") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            confidence_auto_identity_detect = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "motion_vector_screen_enable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            motion_vector_screen_enable = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "motion_vector_screen_min_std_th") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            motion_vector_screen_min_std_th = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "motion_vector_screen_std_factor_th") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            motion_vector_screen_std_factor_th = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "conf1") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            conf1 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "conf2") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            conf2 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "conf3") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            conf3 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "conf4") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            conf4 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "conf5") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            conf5 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "contrast_gain_factor") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            contrast_gain_factor = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "reserved1") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            reserved1 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "reserved2") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            reserved2 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "reserved3") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            reserved3 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "reserved4") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            reserved4 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "reserved5") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            reserved5 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "ica_filtering") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            ica_filtering = (float)dblValue; 
        }
    }

    return __result;
}

bool cvp20_rgn_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // robustness_max_allowed_ncc
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("robustness_max_allowed_ncc", ParameterParserDataType::UInt32, (double)robustness_max_allowed_ncc, 0));
    }
    else if (PARAMETER_STRCMP(__name, "robustness_max_allowed_ncc") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)robustness_max_allowed_ncc, 0));
    }

    // robustness_min_allowed_tar_var
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("robustness_min_allowed_tar_var", ParameterParserDataType::UInt32, (double)robustness_min_allowed_tar_var, 0));
    }
    else if (PARAMETER_STRCMP(__name, "robustness_min_allowed_tar_var") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)robustness_min_allowed_tar_var, 0));
    }

    // robustness_meaningful_ncc_diff
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("robustness_meaningful_ncc_diff", ParameterParserDataType::UInt32, (double)robustness_meaningful_ncc_diff, 0));
    }
    else if (PARAMETER_STRCMP(__name, "robustness_meaningful_ncc_diff") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)robustness_meaningful_ncc_diff, 0));
    }

    // robustness_measure_dist_map_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("robustness_measure_dist_map_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "robustness_measure_dist_map_tab") == 0)
    {
        __result = robustness_measure_dist_map_tab.GetInfo(&__path[__pos], __info);
    }

    // fpx_kappa
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("fpx_kappa", ParameterParserDataType::UInt32, (double)fpx_kappa, 0));
    }
    else if (PARAMETER_STRCMP(__name, "fpx_kappa") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)fpx_kappa, 0));
    }

    // fpx_threshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("fpx_threshold", ParameterParserDataType::UInt32, (double)fpx_threshold, 0));
    }
    else if (PARAMETER_STRCMP(__name, "fpx_threshold") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)fpx_threshold, 0));
    }

    // desc_match_threshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("desc_match_threshold", ParameterParserDataType::UInt32, (double)desc_match_threshold, 0));
    }
    else if (PARAMETER_STRCMP(__name, "desc_match_threshold") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)desc_match_threshold, 0));
    }

    // robustness_threshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("robustness_threshold", ParameterParserDataType::Int32, (double)robustness_threshold, 0));
    }
    else if (PARAMETER_STRCMP(__name, "robustness_threshold") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)robustness_threshold, 0));
    }

    // inlier_track_enable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("inlier_track_enable", ParameterParserDataType::UInt32, (double)inlier_track_enable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "inlier_track_enable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)inlier_track_enable, 0));
    }

    // enable_transform_confidence
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enable_transform_confidence", ParameterParserDataType::UInt32, (double)enable_transform_confidence, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enable_transform_confidence") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)enable_transform_confidence, 0));
    }

    // transform_confidence_mapping_base
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("transform_confidence_mapping_base", ParameterParserDataType::UInt32, (double)transform_confidence_mapping_base, 0));
    }
    else if (PARAMETER_STRCMP(__name, "transform_confidence_mapping_base") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)transform_confidence_mapping_base, 0));
    }

    // transform_confidence_mapping_c1
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("transform_confidence_mapping_c1", ParameterParserDataType::UInt32, (double)transform_confidence_mapping_c1, 0));
    }
    else if (PARAMETER_STRCMP(__name, "transform_confidence_mapping_c1") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)transform_confidence_mapping_c1, 0));
    }

    // transform_confidence_mapping_c2
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("transform_confidence_mapping_c2", ParameterParserDataType::UInt32, (double)transform_confidence_mapping_c2, 0));
    }
    else if (PARAMETER_STRCMP(__name, "transform_confidence_mapping_c2") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)transform_confidence_mapping_c2, 0));
    }

    // transform_confidence_thr_to_force_identity_transform
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("transform_confidence_thr_to_force_identity_transform", ParameterParserDataType::UInt32, (double)transform_confidence_thr_to_force_identity_transform, 0));
    }
    else if (PARAMETER_STRCMP(__name, "transform_confidence_thr_to_force_identity_transform") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)transform_confidence_thr_to_force_identity_transform, 0));
    }

    // transform_model
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("transform_model", ParameterParserDataType::UInt32, (double)transform_model, 0));
    }
    else if (PARAMETER_STRCMP(__name, "transform_model") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)transform_model, 0));
    }

    // model_match_thr
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("model_match_thr", ParameterParserDataType::Float, (double)model_match_thr));
    }
    else if (PARAMETER_STRCMP(__name, "model_match_thr") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)model_match_thr));
    }

    // multi_frame_input_resolution
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("multi_frame_input_resolution", ParameterParserDataType::UInt32, (double)multi_frame_input_resolution, 0));
    }
    else if (PARAMETER_STRCMP(__name, "multi_frame_input_resolution") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)multi_frame_input_resolution, 0));
    }

    // video_registration_down_scale_ratio
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("video_registration_down_scale_ratio", ParameterParserDataType::Float, (double)video_registration_down_scale_ratio));
    }
    else if (PARAMETER_STRCMP(__name, "video_registration_down_scale_ratio") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)video_registration_down_scale_ratio));
    }

    // auto_identity_detect_mode
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("auto_identity_detect_mode", ParameterParserDataType::UInt32, (double)auto_identity_detect_mode, 0));
    }
    else if (PARAMETER_STRCMP(__name, "auto_identity_detect_mode") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)auto_identity_detect_mode, 0));
    }

    // identity_detect_enable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("identity_detect_enable", ParameterParserDataType::UInt32, (double)identity_detect_enable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_enable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)identity_detect_enable, 0));
    }

    // identity_detect_consistency_mesure_th
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("identity_detect_consistency_mesure_th", ParameterParserDataType::UInt32, (double)identity_detect_consistency_mesure_th, 0));
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_consistency_mesure_th") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)identity_detect_consistency_mesure_th, 0));
    }

    // identity_detection_mvs_avg_size_th
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("identity_detection_mvs_avg_size_th", ParameterParserDataType::Float, (double)identity_detection_mvs_avg_size_th));
    }
    else if (PARAMETER_STRCMP(__name, "identity_detection_mvs_avg_size_th") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)identity_detection_mvs_avg_size_th));
    }

    // identity_detect_big_mvs_percent_th
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("identity_detect_big_mvs_percent_th", ParameterParserDataType::UInt32, (double)identity_detect_big_mvs_percent_th, 0));
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_big_mvs_percent_th") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)identity_detect_big_mvs_percent_th, 0));
    }

    // identity_detect_temporal_consistency
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("identity_detect_temporal_consistency", ParameterParserDataType::UInt32, (double)identity_detect_temporal_consistency, 0));
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_temporal_consistency") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)identity_detect_temporal_consistency, 0));
    }

    // identity_detect_complexity_th
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("identity_detect_complexity_th", ParameterParserDataType::UInt32, (double)identity_detect_complexity_th, 0));
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_complexity_th") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)identity_detect_complexity_th, 0));
    }

    // identity_detect_max_num_mvs_th
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("identity_detect_max_num_mvs_th", ParameterParserDataType::UInt32, (double)identity_detect_max_num_mvs_th, 0));
    }
    else if (PARAMETER_STRCMP(__name, "identity_detect_max_num_mvs_th") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)identity_detect_max_num_mvs_th, 0));
    }

    // confidence_auto_identity_detect
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("confidence_auto_identity_detect", ParameterParserDataType::UInt32, (double)confidence_auto_identity_detect, 0));
    }
    else if (PARAMETER_STRCMP(__name, "confidence_auto_identity_detect") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)confidence_auto_identity_detect, 0));
    }

    // motion_vector_screen_enable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("motion_vector_screen_enable", ParameterParserDataType::UInt32, (double)motion_vector_screen_enable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "motion_vector_screen_enable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)motion_vector_screen_enable, 0));
    }

    // motion_vector_screen_min_std_th
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("motion_vector_screen_min_std_th", ParameterParserDataType::Float, (double)motion_vector_screen_min_std_th));
    }
    else if (PARAMETER_STRCMP(__name, "motion_vector_screen_min_std_th") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)motion_vector_screen_min_std_th));
    }

    // motion_vector_screen_std_factor_th
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("motion_vector_screen_std_factor_th", ParameterParserDataType::Float, (double)motion_vector_screen_std_factor_th));
    }
    else if (PARAMETER_STRCMP(__name, "motion_vector_screen_std_factor_th") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)motion_vector_screen_std_factor_th));
    }

    // conf1
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("conf1", ParameterParserDataType::Float, (double)conf1));
    }
    else if (PARAMETER_STRCMP(__name, "conf1") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)conf1));
    }

    // conf2
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("conf2", ParameterParserDataType::Float, (double)conf2));
    }
    else if (PARAMETER_STRCMP(__name, "conf2") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)conf2));
    }

    // conf3
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("conf3", ParameterParserDataType::Float, (double)conf3));
    }
    else if (PARAMETER_STRCMP(__name, "conf3") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)conf3));
    }

    // conf4
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("conf4", ParameterParserDataType::Float, (double)conf4));
    }
    else if (PARAMETER_STRCMP(__name, "conf4") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)conf4));
    }

    // conf5
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("conf5", ParameterParserDataType::Float, (double)conf5));
    }
    else if (PARAMETER_STRCMP(__name, "conf5") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)conf5));
    }

    // contrast_gain_factor
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("contrast_gain_factor", ParameterParserDataType::Float, (double)contrast_gain_factor));
    }
    else if (PARAMETER_STRCMP(__name, "contrast_gain_factor") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)contrast_gain_factor));
    }

    // reserved1
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("reserved1", ParameterParserDataType::Float, (double)reserved1));
    }
    else if (PARAMETER_STRCMP(__name, "reserved1") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)reserved1));
    }

    // reserved2
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("reserved2", ParameterParserDataType::Float, (double)reserved2));
    }
    else if (PARAMETER_STRCMP(__name, "reserved2") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)reserved2));
    }

    // reserved3
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("reserved3", ParameterParserDataType::Float, (double)reserved3));
    }
    else if (PARAMETER_STRCMP(__name, "reserved3") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)reserved3));
    }

    // reserved4
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("reserved4", ParameterParserDataType::Float, (double)reserved4));
    }
    else if (PARAMETER_STRCMP(__name, "reserved4") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)reserved4));
    }

    // reserved5
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("reserved5", ParameterParserDataType::Float, (double)reserved5));
    }
    else if (PARAMETER_STRCMP(__name, "reserved5") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)reserved5));
    }

    // ica_filtering
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ica_filtering", ParameterParserDataType::Float, (double)ica_filtering));
    }
    else if (PARAMETER_STRCMP(__name, "ica_filtering") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)ica_filtering));
    }

    return __result;
}

bool cvp20_rgn_dataType::robustness_measure_dist_map_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "robustness_measure_dist_map") == 0)
    {
        if (__index < 8)
        {
            ParameterUtils::DoubleToString((double)robustness_measure_dist_map[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 8; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)robustness_measure_dist_map[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }

    return __result;
}

bool cvp20_rgn_dataType::robustness_measure_dist_map_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "robustness_measure_dist_map") == 0)
    {
        if (__index < 8)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                robustness_measure_dist_map[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 8)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    robustness_measure_dist_map[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool cvp20_rgn_dataType::robustness_measure_dist_map_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // robustness_measure_dist_map
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("robustness_measure_dist_map", ParameterParserDataType::UInt32, true, false, 8));
    }

    return __result;
}

bool mod_cvp20_aec_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "cvp20_rgn_data") == 0)
    {
        __result = cvp20_rgn_data.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool mod_cvp20_aec_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "cvp20_rgn_data") == 0)
    {
        __result = cvp20_rgn_data.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool mod_cvp20_aec_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // aec_trigger
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("aec_trigger", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.GetInfo(&__path[__pos], __info);
    }

    // cvp20_rgn_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("cvp20_rgn_data", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "cvp20_rgn_data") == 0)
    {
        __result = cvp20_rgn_data.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool mod_cvp20_hdr_aec_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "hdr_aec_trigger") == 0)
    {
        __result = hdr_aec_trigger.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "hdr_aec_data") == 0)
    {
        __result = hdr_aec_data.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool mod_cvp20_hdr_aec_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "hdr_aec_trigger") == 0)
    {
        __result = hdr_aec_trigger.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "hdr_aec_data") == 0)
    {
        __result = hdr_aec_data.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool mod_cvp20_hdr_aec_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // hdr_aec_trigger
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hdr_aec_trigger", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "hdr_aec_trigger") == 0)
    {
        __result = hdr_aec_trigger.GetInfo(&__path[__pos], __info);
    }

    // hdr_aec_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hdr_aec_data", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "hdr_aec_data") == 0)
    {
        __result = hdr_aec_data.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool mod_cvp20_hdr_aec_dataType::hdr_aec_dataStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_cvp20_aec_data") == 0)
    {
        if (__index < mod_cvp20_aec_dataCount)
        {
            __result = mod_cvp20_aec_data[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool mod_cvp20_hdr_aec_dataType::hdr_aec_dataStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_cvp20_aec_data") == 0)
    {
        if (__index < mod_cvp20_aec_dataCount)
        {
            __result = mod_cvp20_aec_data[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool mod_cvp20_hdr_aec_dataType::hdr_aec_dataStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // mod_cvp20_aec_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("mod_cvp20_aec_data", ParameterParserDataType::Structure, true, false, mod_cvp20_aec_dataCount));
    }
    else if (PARAMETER_STRCMP(__name, "mod_cvp20_aec_data") == 0)
    {
        if (__index < mod_cvp20_aec_dataCount)
        {
            __result = mod_cvp20_aec_data[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, mod_cvp20_aec_dataCount));
        }
    }

    return __result;
}

bool mod_cvp20_pre_scale_ratio_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "pre_scale_ratio_trigger") == 0)
    {
        __result = pre_scale_ratio_trigger.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "pre_scale_ratio_data") == 0)
    {
        __result = pre_scale_ratio_data.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool mod_cvp20_pre_scale_ratio_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "pre_scale_ratio_trigger") == 0)
    {
        __result = pre_scale_ratio_trigger.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "pre_scale_ratio_data") == 0)
    {
        __result = pre_scale_ratio_data.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool mod_cvp20_pre_scale_ratio_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // pre_scale_ratio_trigger
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("pre_scale_ratio_trigger", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "pre_scale_ratio_trigger") == 0)
    {
        __result = pre_scale_ratio_trigger.GetInfo(&__path[__pos], __info);
    }

    // pre_scale_ratio_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("pre_scale_ratio_data", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "pre_scale_ratio_data") == 0)
    {
        __result = pre_scale_ratio_data.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool mod_cvp20_pre_scale_ratio_dataType::pre_scale_ratio_dataStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_cvp20_hdr_aec_data") == 0)
    {
        if (__index < mod_cvp20_hdr_aec_dataCount)
        {
            __result = mod_cvp20_hdr_aec_data[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool mod_cvp20_pre_scale_ratio_dataType::pre_scale_ratio_dataStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_cvp20_hdr_aec_data") == 0)
    {
        if (__index < mod_cvp20_hdr_aec_dataCount)
        {
            __result = mod_cvp20_hdr_aec_data[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool mod_cvp20_pre_scale_ratio_dataType::pre_scale_ratio_dataStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // mod_cvp20_hdr_aec_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("mod_cvp20_hdr_aec_data", ParameterParserDataType::Structure, true, false, mod_cvp20_hdr_aec_dataCount));
    }
    else if (PARAMETER_STRCMP(__name, "mod_cvp20_hdr_aec_data") == 0)
    {
        if (__index < mod_cvp20_hdr_aec_dataCount)
        {
            __result = mod_cvp20_hdr_aec_data[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, mod_cvp20_hdr_aec_dataCount));
        }
    }

    return __result;
}

bool chromatix_cvp20_coreType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_cvp20_pre_scale_ratio_data") == 0)
    {
        if (__index < mod_cvp20_pre_scale_ratio_dataCount)
        {
            __result = mod_cvp20_pre_scale_ratio_data[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool chromatix_cvp20_coreType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_cvp20_pre_scale_ratio_data") == 0)
    {
        if (__index < mod_cvp20_pre_scale_ratio_dataCount)
        {
            __result = mod_cvp20_pre_scale_ratio_data[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool chromatix_cvp20_coreType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // mod_cvp20_pre_scale_ratio_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("mod_cvp20_pre_scale_ratio_data", ParameterParserDataType::Structure, true, false, mod_cvp20_pre_scale_ratio_dataCount));
    }
    else if (PARAMETER_STRCMP(__name, "mod_cvp20_pre_scale_ratio_data") == 0)
    {
        if (__index < mod_cvp20_pre_scale_ratio_dataCount)
        {
            __result = mod_cvp20_pre_scale_ratio_data[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, mod_cvp20_pre_scale_ratio_dataCount));
        }
    }

    return __result;
}

bool chromatix_cvp20Type::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_cvp20_reserve") == 0)
    {
        __result = chromatix_cvp20_reserve.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_cvp20_core") == 0)
    {
        __result = chromatix_cvp20_core.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool chromatix_cvp20Type::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_cvp20_reserve") == 0)
    {
        __result = chromatix_cvp20_reserve.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_cvp20_core") == 0)
    {
        __result = chromatix_cvp20_core.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool chromatix_cvp20Type::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // module_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("module_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetInfo(&__path[__pos], __info);
    }

    // parameter_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("parameter_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.GetInfo(&__path[__pos], __info);
    }

    // control_variables
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_variables", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.GetInfo(&__path[__pos], __info);
    }

    // control_method
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_method", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.GetInfo(&__path[__pos], __info);
    }

    // chromatix_cvp20_reserve
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("chromatix_cvp20_reserve", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_cvp20_reserve") == 0)
    {
        __result = chromatix_cvp20_reserve.GetInfo(&__path[__pos], __info);
    }

    // chromatix_cvp20_core
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("chromatix_cvp20_core", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_cvp20_core") == 0)
    {
        __result = chromatix_cvp20_core.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatix_cvp20Type::module_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)major_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)minor_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)incr_revision, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_cvp20Type::module_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            major_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minor_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            incr_revision = (INT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_cvp20Type::module_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // major_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("major_revision", ParameterParserDataType::Int32, (double)major_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)major_revision, 0));
    }

    // minor_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minor_revision", ParameterParserDataType::Int32, (double)minor_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)minor_revision, 0));
    }

    // incr_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("incr_revision", ParameterParserDataType::Int32, (double)incr_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)incr_revision, 0));
    }

    return __result;
}

bool chromatix_cvp20Type::parameter_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        PARAMETER_STRCPY(__value, __size, revision);
        __result = true;
    }

    return __result;
}

bool chromatix_cvp20Type::parameter_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const /*__value*/)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        // Cannot modify strings, risk of memory leaks or corruption
    }

    return __result;
}

bool chromatix_cvp20Type::parameter_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("revision", ParameterParserDataType::String, revision));
    }
    else if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::String, revision));
    }

    return __result;
}

bool chromatix_cvp20Type::control_variablesStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "control_var_type") == 0)
    {
        if (__index < 3)
        {
            ParameterUtils::DoubleToString((double)(UINT32)control_var_type[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 3; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)(UINT32)control_var_type[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }

    return __result;
}

bool chromatix_cvp20Type::control_variablesStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "control_var_type") == 0)
    {
        if (__index < 3)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                control_var_type[__index] = (control_var_typeEnum)(UINT32)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 3)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    control_var_type[__index] = (control_var_typeEnum)(UINT32)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool chromatix_cvp20Type::control_variablesStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // control_var_type
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_var_type", ParameterParserDataType::Enum, true, false, 3));
    }

    return __result;
}

bool chromatix_cvp20Type::control_methodStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)aec_exp_control, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "aec_hdr_control") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)aec_hdr_control, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_cvp20Type::control_methodStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            aec_exp_control = (ispglobalelements::tuning_control_aec_type)(UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "aec_hdr_control") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            aec_hdr_control = (ispglobalelements::tuning_control_aec_hdr_type)(UINT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_cvp20Type::control_methodStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // aec_exp_control
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("aec_exp_control", ParameterParserDataType::Enum, (double)(UINT32)aec_exp_control, 0));
    }
    else if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)aec_exp_control, 0));
    }

    // aec_hdr_control
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("aec_hdr_control", ParameterParserDataType::Enum, (double)(UINT32)aec_hdr_control, 0));
    }
    else if (PARAMETER_STRCMP(__name, "aec_hdr_control") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)aec_hdr_control, 0));
    }

    return __result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_cvp20Type::control_variablesStruct::ReadEnum_control_var_typeEnum(
    ParameterFileSymbolTableEntry* const entry,
    chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum::control_lux_idx):
            enumValue = chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum::control_lux_idx;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum::control_gain):
            enumValue = chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum::control_gain;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum::control_exp_time_ratio):
            enumValue = chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum::control_exp_time_ratio;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum::control_aec_exp_sensitivity_ratio):
            enumValue = chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum::control_aec_exp_sensitivity_ratio;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum::control_exp_gain_ratio):
            enumValue = chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum::control_exp_gain_ratio;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum::control_pre_scale_ratio):
            enumValue = chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum::control_pre_scale_ratio;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to chromatix_cvp20Type::control_variablesStruct::control_var_typeEnum", uintValue);
            break;
        }
    }
    return result;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Default Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_cvp20TypeClass::chromatix_cvp20TypeClass(
    const CHAR* const name)
    : ParameterModule(name, MAJOR, MINOR, PATCH, nullptr)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(chromatix_cvp20_reserve);
    PARAMETER_INITIALIZE(chromatix_cvp20_core);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_cvp20TypeClass::chromatix_cvp20TypeClass(
    const CHAR* const name,
    ModuleData* const moduleData)
    : ParameterModule(name, MAJOR, MINOR, PATCH, moduleData)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(chromatix_cvp20_reserve);
    PARAMETER_INITIALIZE(chromatix_cvp20_core);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_cvp20TypeClass::~chromatix_cvp20TypeClass()
{
    if (Loaded)
    {
        UnloadModule();
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// GetName
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const CHAR* chromatix_cvp20TypeClass::GetName(
    const CHAR* const type)
{
    const CHAR* name = type;
    CHAR temp[ParameterFileSymbolTableEntry::TYPE_LEN + 1];
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "cvp20_ipe_mac")) == 0)
    {
        name = "cvp20_ipe_mac";
    }
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "cvp20_ipe_temporal")) == 0)
    {
        name = "cvp20_ipe_temporal";
    }
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "cvp20_ipe_sat")) == 0)
    {
        name = "cvp20_ipe_sat";
    }
    return name;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_cvp20_reserveType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_cvp20_reserveType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(image_based_alignment_enable);
    PARAMETER_INITIALIZE(descriptor_lpf);

    SYMBOL_TABLE_ENTRY_READ(entry, result, image_based_alignment_enable, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, descriptor_lpf, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load cvp20_rgn_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool cvp20_rgn_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(robustness_max_allowed_ncc);
    PARAMETER_INITIALIZE(robustness_min_allowed_tar_var);
    PARAMETER_INITIALIZE(robustness_meaningful_ncc_diff);
    PARAMETER_INITIALIZE(robustness_measure_dist_map_tab);
    PARAMETER_INITIALIZE(fpx_kappa);
    PARAMETER_INITIALIZE(fpx_threshold);
    PARAMETER_INITIALIZE(desc_match_threshold);
    PARAMETER_INITIALIZE(robustness_threshold);
    PARAMETER_INITIALIZE(inlier_track_enable);
    PARAMETER_INITIALIZE(enable_transform_confidence);
    PARAMETER_INITIALIZE(transform_confidence_mapping_base);
    PARAMETER_INITIALIZE(transform_confidence_mapping_c1);
    PARAMETER_INITIALIZE(transform_confidence_mapping_c2);
    PARAMETER_INITIALIZE(transform_confidence_thr_to_force_identity_transform);
    PARAMETER_INITIALIZE(transform_model);
    PARAMETER_INITIALIZE(model_match_thr);
    PARAMETER_INITIALIZE(multi_frame_input_resolution);
    PARAMETER_INITIALIZE(video_registration_down_scale_ratio);
    PARAMETER_INITIALIZE(auto_identity_detect_mode);
    PARAMETER_INITIALIZE(identity_detect_enable);
    PARAMETER_INITIALIZE(identity_detect_consistency_mesure_th);
    PARAMETER_INITIALIZE(identity_detection_mvs_avg_size_th);
    PARAMETER_INITIALIZE(identity_detect_big_mvs_percent_th);
    PARAMETER_INITIALIZE(identity_detect_temporal_consistency);
    PARAMETER_INITIALIZE(identity_detect_complexity_th);
    PARAMETER_INITIALIZE(identity_detect_max_num_mvs_th);
    PARAMETER_INITIALIZE(confidence_auto_identity_detect);
    PARAMETER_INITIALIZE(motion_vector_screen_enable);
    PARAMETER_INITIALIZE(motion_vector_screen_min_std_th);
    PARAMETER_INITIALIZE(motion_vector_screen_std_factor_th);
    PARAMETER_INITIALIZE(conf1);
    PARAMETER_INITIALIZE(conf2);
    PARAMETER_INITIALIZE(conf3);
    PARAMETER_INITIALIZE(conf4);
    PARAMETER_INITIALIZE(conf5);
    PARAMETER_INITIALIZE(contrast_gain_factor);
    PARAMETER_INITIALIZE(reserved1);
    PARAMETER_INITIALIZE(reserved2);
    PARAMETER_INITIALIZE(reserved3);
    PARAMETER_INITIALIZE(reserved4);
    PARAMETER_INITIALIZE(reserved5);
    PARAMETER_INITIALIZE(ica_filtering);

    SYMBOL_TABLE_ENTRY_READ(entry, result, robustness_max_allowed_ncc, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, robustness_min_allowed_tar_var, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, robustness_meaningful_ncc_diff, alignment)
    if (result)
    {
        result = robustness_measure_dist_map_tab.Load(entry, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, fpx_kappa, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, fpx_threshold, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, desc_match_threshold, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, robustness_threshold, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, inlier_track_enable, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, enable_transform_confidence, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, transform_confidence_mapping_base, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, transform_confidence_mapping_c1, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, transform_confidence_mapping_c2, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, transform_confidence_thr_to_force_identity_transform, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, transform_model, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, model_match_thr, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, multi_frame_input_resolution, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, video_registration_down_scale_ratio, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, auto_identity_detect_mode, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, identity_detect_enable, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, identity_detect_consistency_mesure_th, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, identity_detection_mvs_avg_size_th, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, identity_detect_big_mvs_percent_th, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, identity_detect_temporal_consistency, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, identity_detect_complexity_th, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, identity_detect_max_num_mvs_th, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, confidence_auto_identity_detect, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, motion_vector_screen_enable, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, motion_vector_screen_min_std_th, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, motion_vector_screen_std_factor_th, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, conf1, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, conf2, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, conf3, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, conf4, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, conf5, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, contrast_gain_factor, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, reserved1, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, reserved2, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, reserved3, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, reserved4, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, reserved5, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, ica_filtering, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load cvp20_rgn_dataType_robustness_measure_dist_map_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool cvp20_rgn_dataType::robustness_measure_dist_map_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(robustness_measure_dist_map);

    result = entry->ReadArray(robustness_measure_dist_map, 8, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_cvp20_aec_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_cvp20_aec_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(aec_trigger);
    PARAMETER_INITIALIZE(cvp20_rgn_data);

    result = aec_trigger.Load(entry, alignment);
    if (result)
    {
        result = cvp20_rgn_data.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_cvp20_hdr_aec_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_cvp20_hdr_aec_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(hdr_aec_trigger);
    PARAMETER_INITIALIZE(hdr_aec_data);

    result = hdr_aec_trigger.Load(entry, alignment);
    if (result)
    {
        result = hdr_aec_data.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_cvp20_hdr_aec_dataType_hdr_aec_dataStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_cvp20_hdr_aec_dataType::hdr_aec_dataStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(mod_cvp20_aec_data);


    // Reading Array mod_cvp20_aec_data from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        mod_cvp20_aec_dataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, mod_cvp20_aec_dataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            mod_cvp20_aec_dataID = pointerEntry->ID;
            DupObjInfo* mod_cvp20_aec_dataDupInfo = pointerEntry->GetDupObject(mod_cvp20_aec_dataCount, file, line);
            if (mod_cvp20_aec_dataDupInfo != nullptr && mod_cvp20_aec_dataDupInfo->Object != nullptr)
            {
                mod_cvp20_aec_data = (mod_cvp20_aec_dataType*)mod_cvp20_aec_dataDupInfo->Object;
                if (mod_cvp20_aec_data == nullptr)
                {
                    if (mod_cvp20_aec_dataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_cvp20_aec_data duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            mod_cvp20_aec_dataDupInfo->File, mod_cvp20_aec_dataDupInfo->Line, mod_cvp20_aec_dataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_cvp20_aec_data duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            mod_cvp20_aec_dataDupInfo->Line, mod_cvp20_aec_dataDupInfo->RefCount);
                    }
                }
            }

            if (mod_cvp20_aec_data == nullptr)
            {
                if (result && mod_cvp20_aec_dataCount > 0)
                {
                    mod_cvp20_aec_data = PARAMETER_NEW mod_cvp20_aec_dataType[mod_cvp20_aec_dataCount];
                    (void)pointerEntry->SetDupObject(mod_cvp20_aec_data, mod_cvp20_aec_dataCount, file, line);
                }
                if (!result || mod_cvp20_aec_data == nullptr)
                {
                    if (mod_cvp20_aec_dataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate mod_cvp20_aec_data\n");
                    }
                    mod_cvp20_aec_dataCount = 0;
                }
                for (UINT32 i = 0; result && i < mod_cvp20_aec_dataCount; i++)
                {
                    result = mod_cvp20_aec_data[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (mod_cvp20_aec_data == nullptr)
        {
            mod_cvp20_aec_dataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_cvp20_pre_scale_ratio_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_cvp20_pre_scale_ratio_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(pre_scale_ratio_trigger);
    PARAMETER_INITIALIZE(pre_scale_ratio_data);

    result = pre_scale_ratio_trigger.Load(entry, alignment);
    if (result)
    {
        result = pre_scale_ratio_data.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_cvp20_pre_scale_ratio_dataType_pre_scale_ratio_dataStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_cvp20_pre_scale_ratio_dataType::pre_scale_ratio_dataStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(mod_cvp20_hdr_aec_data);


    // Reading Array mod_cvp20_hdr_aec_data from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        mod_cvp20_hdr_aec_dataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, mod_cvp20_hdr_aec_dataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            mod_cvp20_hdr_aec_dataID = pointerEntry->ID;
            DupObjInfo* mod_cvp20_hdr_aec_dataDupInfo = pointerEntry->GetDupObject(mod_cvp20_hdr_aec_dataCount, file, line);
            if (mod_cvp20_hdr_aec_dataDupInfo != nullptr && mod_cvp20_hdr_aec_dataDupInfo->Object != nullptr)
            {
                mod_cvp20_hdr_aec_data = (mod_cvp20_hdr_aec_dataType*)mod_cvp20_hdr_aec_dataDupInfo->Object;
                if (mod_cvp20_hdr_aec_data == nullptr)
                {
                    if (mod_cvp20_hdr_aec_dataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_cvp20_hdr_aec_data duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            mod_cvp20_hdr_aec_dataDupInfo->File, mod_cvp20_hdr_aec_dataDupInfo->Line, mod_cvp20_hdr_aec_dataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_cvp20_hdr_aec_data duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            mod_cvp20_hdr_aec_dataDupInfo->Line, mod_cvp20_hdr_aec_dataDupInfo->RefCount);
                    }
                }
            }

            if (mod_cvp20_hdr_aec_data == nullptr)
            {
                if (result && mod_cvp20_hdr_aec_dataCount > 0)
                {
                    mod_cvp20_hdr_aec_data = PARAMETER_NEW mod_cvp20_hdr_aec_dataType[mod_cvp20_hdr_aec_dataCount];
                    (void)pointerEntry->SetDupObject(mod_cvp20_hdr_aec_data, mod_cvp20_hdr_aec_dataCount, file, line);
                }
                if (!result || mod_cvp20_hdr_aec_data == nullptr)
                {
                    if (mod_cvp20_hdr_aec_dataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate mod_cvp20_hdr_aec_data\n");
                    }
                    mod_cvp20_hdr_aec_dataCount = 0;
                }
                for (UINT32 i = 0; result && i < mod_cvp20_hdr_aec_dataCount; i++)
                {
                    result = mod_cvp20_hdr_aec_data[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (mod_cvp20_hdr_aec_data == nullptr)
        {
            mod_cvp20_hdr_aec_dataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_cvp20_coreType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_cvp20_coreType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(mod_cvp20_pre_scale_ratio_data);


    // Reading Array mod_cvp20_pre_scale_ratio_data from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        mod_cvp20_pre_scale_ratio_dataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, mod_cvp20_pre_scale_ratio_dataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            mod_cvp20_pre_scale_ratio_dataID = pointerEntry->ID;
            DupObjInfo* mod_cvp20_pre_scale_ratio_dataDupInfo = pointerEntry->GetDupObject(mod_cvp20_pre_scale_ratio_dataCount, file, line);
            if (mod_cvp20_pre_scale_ratio_dataDupInfo != nullptr && mod_cvp20_pre_scale_ratio_dataDupInfo->Object != nullptr)
            {
                mod_cvp20_pre_scale_ratio_data = (mod_cvp20_pre_scale_ratio_dataType*)mod_cvp20_pre_scale_ratio_dataDupInfo->Object;
                if (mod_cvp20_pre_scale_ratio_data == nullptr)
                {
                    if (mod_cvp20_pre_scale_ratio_dataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_cvp20_pre_scale_ratio_data duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            mod_cvp20_pre_scale_ratio_dataDupInfo->File, mod_cvp20_pre_scale_ratio_dataDupInfo->Line, mod_cvp20_pre_scale_ratio_dataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_cvp20_pre_scale_ratio_data duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            mod_cvp20_pre_scale_ratio_dataDupInfo->Line, mod_cvp20_pre_scale_ratio_dataDupInfo->RefCount);
                    }
                }
            }

            if (mod_cvp20_pre_scale_ratio_data == nullptr)
            {
                if (result && mod_cvp20_pre_scale_ratio_dataCount > 0)
                {
                    mod_cvp20_pre_scale_ratio_data = PARAMETER_NEW mod_cvp20_pre_scale_ratio_dataType[mod_cvp20_pre_scale_ratio_dataCount];
                    (void)pointerEntry->SetDupObject(mod_cvp20_pre_scale_ratio_data, mod_cvp20_pre_scale_ratio_dataCount, file, line);
                }
                if (!result || mod_cvp20_pre_scale_ratio_data == nullptr)
                {
                    if (mod_cvp20_pre_scale_ratio_dataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate mod_cvp20_pre_scale_ratio_data\n");
                    }
                    mod_cvp20_pre_scale_ratio_dataCount = 0;
                }
                for (UINT32 i = 0; result && i < mod_cvp20_pre_scale_ratio_dataCount; i++)
                {
                    result = mod_cvp20_pre_scale_ratio_data[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (mod_cvp20_pre_scale_ratio_data == nullptr)
        {
            mod_cvp20_pre_scale_ratio_dataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_cvp20Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_cvp20Type::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(chromatix_cvp20_reserve);
    PARAMETER_INITIALIZE(chromatix_cvp20_core);

    result = module_version.Load(entry, alignment);
    if (result)
    {
        result = parameter_version.Load(entry, alignment);
    }
    if (result)
    {
        result = control_variables.Load(entry, alignment);
    }
    if (result)
    {
        result = control_method.Load(entry, alignment);
    }
    if (result)
    {
        result = chromatix_cvp20_reserve.Load(entry, alignment);
    }
    if (result)
    {
        result = chromatix_cvp20_core.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_cvp20Type::module_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(major_revision);
    PARAMETER_INITIALIZE(minor_revision);
    PARAMETER_INITIALIZE(incr_revision);

    SYMBOL_TABLE_ENTRY_READ(entry, result, major_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, minor_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, incr_revision, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load parameter_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_cvp20Type::parameter_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(revision);


    // Reading String revision from symbol table
    {
        UINT32 count = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, count, alignment)
        ParameterFileSymbolTableEntry stringPointerEntryCls;
        ParameterFileSymbolTableEntry* const stringPointerEntry = &stringPointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(stringPointerEntryCls, entry, alignment);
        }
        SYMBOL_TABLE_ENTRY_READ_STRING(stringPointerEntry, result, revision, count, alignment)
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load control_variablesStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_cvp20Type::control_variablesStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(control_var_type);

    for (UINT32 i = 0; result && i < 3; i++)
    {
        result = chromatix_cvp20Type::control_variablesStruct::ReadEnum_control_var_typeEnum(entry, control_var_type[i], alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load control_methodStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_cvp20Type::control_methodStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(aec_exp_control);
    PARAMETER_INITIALIZE(aec_hdr_control);

    result = ispglobalelements::ReadEnum_tuning_control_aec_type(entry, aec_exp_control, alignment);
    if (result)
    {
        result = ispglobalelements::ReadEnum_tuning_control_aec_hdr_type(entry, aec_hdr_control, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload mod_cvp20_hdr_aec_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void mod_cvp20_hdr_aec_dataType::Unload(ParameterFileSymbolTable* const __table)
{
    hdr_aec_data.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload mod_cvp20_hdr_aec_dataType_hdr_aec_dataStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void mod_cvp20_hdr_aec_dataType::hdr_aec_dataStruct::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(mod_cvp20_aec_data))
    {
        PARAMETER_DELETE[] mod_cvp20_aec_data;
    }
    mod_cvp20_aec_data = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload mod_cvp20_pre_scale_ratio_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void mod_cvp20_pre_scale_ratio_dataType::Unload(ParameterFileSymbolTable* const __table)
{
    pre_scale_ratio_data.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload mod_cvp20_pre_scale_ratio_dataType_pre_scale_ratio_dataStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void mod_cvp20_pre_scale_ratio_dataType::pre_scale_ratio_dataStruct::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < mod_cvp20_hdr_aec_dataCount; i++)
    {
        mod_cvp20_hdr_aec_data[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(mod_cvp20_hdr_aec_data))
    {
        PARAMETER_DELETE[] mod_cvp20_hdr_aec_data;
    }
    mod_cvp20_hdr_aec_data = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload chromatix_cvp20_coreType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_cvp20_coreType::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < mod_cvp20_pre_scale_ratio_dataCount; i++)
    {
        mod_cvp20_pre_scale_ratio_data[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(mod_cvp20_pre_scale_ratio_data))
    {
        PARAMETER_DELETE[] mod_cvp20_pre_scale_ratio_data;
    }
    mod_cvp20_pre_scale_ratio_data = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload chromatix_cvp20Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_cvp20Type::Unload(ParameterFileSymbolTable* const __table)
{
    parameter_version.Unload(__table);
    chromatix_cvp20_core.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload parameter_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_cvp20Type::parameter_versionStruct::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(revision))
    {
        PARAMETER_DELETE[] revision;
    }
    revision = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Release the ParameterModule object
/// Requires that this object was created via a ParameterSetManager::GetModule call
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_cvp20Type::Release()
{
    ((chromatix_cvp20TypeClass*)this)->Release();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Returns the ParameterModule::Modified value
/// The Modified value is incremented when the module's data or pointer values change
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
UINT32 chromatix_cvp20Type::GetModified()
{
    return ((chromatix_cvp20TypeClass*)this)->Modified;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Create module, and return a new object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
ParameterModule* chromatix_cvp20TypeClass::Create(
    ModuleData*    moduleData,
    DefaultModule* const defModule) const
{
    chromatix_cvp20TypeClass* cls = nullptr;

    if (PARAMETER_STRCMP(Type, moduleData->Type) == 0 && Version.Value == moduleData->Version.Value)
    {
        const UINT32 id = moduleData->DuplicateID > 0 ? moduleData->DuplicateID : moduleData->ID;

        ParameterModule* const module = defModule->Find(id);
        if (module != nullptr)
        {
            cls = (chromatix_cvp20TypeClass*)module;
        }
        else
        {
            cls = PARAMETER_NEW chromatix_cvp20TypeClass(GetName(moduleData->Type), moduleData);
            cls->SymbolTableID = id;
            defModule->AddModule((ParameterModule*)cls);
        }
    }
    else if (Version.Value != moduleData->Version.Value)
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatix_cvp20TypeClass, binary version (%lld) does not match source version (%lld)\n",
            moduleData->Version.Value, Version.Value);
    }
    else
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatix_cvp20TypeClass, binary name (%s) does not match source name (%s)\n",
            moduleData->Type, Type);
    }

    return (ParameterModule*)cls;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module data, return true is successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_cvp20TypeClass::LoadModule(
    ParameterFileSymbolTable* const table)
{
    bool result = true;

    if (!Loaded)
    {
        BYTE const alignment = table->Alignment;
        chromatix_cvp20TypeClass* const cls = this;

        ParameterFileSymbolTableEntry  entryObj;
        ParameterFileSymbolTableEntry* const entry = &entryObj;
        // With EzTune enabled, the unused upper byte of SymbolTableID value can change, so clear it
        result = table->Find(entryObj, SymbolTableID & 0x00ffffff);

        if (!result)
        {
            PARAMETER_LOG_ERROR(entry->SetManager, "chromatix_cvp20TypeClass::LoadModule Failed to find module ID (%d) in Symbol Table\n", SymbolTableID);
        }

        if (result)
        {
            result = cls->Load(entry, alignment);
        }
        Modified++;
        Loaded = true;
    }

    if (!result)
    {
        /// Module failed to fully load, unload it
        UnloadModule();
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload module data
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_cvp20TypeClass::UnloadModule()
{
    if (Loaded)
    {
        if (Data != nullptr && Data->SymbolTable != nullptr)
        {
            Unload(Data->SymbolTable);
        }
        else
        {
            PARAMETER_LOG_ERROR(Data != nullptr ? Data->SetManager : nullptr, "UnloadModule %s called with nullptr "
                "Data->SymbolTable object\n", Name);
            Unload(nullptr);
        }
        Loaded = false;
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// /// Release the ParameterModule object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_cvp20TypeClass::Release()
{
    ParameterModule::Release();
}

CVP_2_0_0_NAMESPACE_END
